@@ -39,7 +39,7 @@ typedef struct {
     uint8_t sample_buf[PCSPK_BUF_LEN];
     QEMUSoundCard card;
     SWVoiceOut *voice;
-    PITState *pit;
+    ISADevice *pit;
     unsigned int pit_count;
     unsigned int samples;
     unsigned int play_pos;
@@ -51,13 +51,14 @@ static const char *s_spk = "pcspk";
 static PCSpkState pcspk_state;
 
 #ifdef CONFIG_KVM_PIT
-static void kvm_get_pit_ch2(PITState *pit,
+static void kvm_get_pit_ch2(ISADevice *dev,
                             struct kvm_pit_state *inkernel_state)
 {
+    struct PITState *pit = DO_UPCAST(struct PITState, dev, dev);
     struct kvm_pit_state pit_state;
 
-    if (kvm_enabled() && qemu_kvm_pit_in_kernel()) {
-        kvm_get_pit(kvm_context, &pit_state);
+    if (kvm_enabled() && kvm_pit_in_kernel()) {
+        kvm_get_pit(kvm_state, &pit_state);
         pit->channels[2].mode = pit_state.channels[2].mode;
         pit->channels[2].count = pit_state.channels[2].count;
         pit->channels[2].count_load_time = pit_state.channels[2].count_load_time;
@@ -68,22 +69,24 @@ static void kvm_get_pit_ch2(PITState *pit,
     }
 }
 
-static void kvm_set_pit_ch2(PITState *pit,
+static void kvm_set_pit_ch2(ISADevice *dev,
                             struct kvm_pit_state *inkernel_state)
 {
-    if (kvm_enabled() && qemu_kvm_pit_in_kernel()) {
+    struct PITState *pit = DO_UPCAST(struct PITState, dev, dev);
+
+    if (kvm_enabled() && kvm_pit_in_kernel()) {
         inkernel_state->channels[2].mode = pit->channels[2].mode;
         inkernel_state->channels[2].count = pit->channels[2].count;
         inkernel_state->channels[2].count_load_time =
             pit->channels[2].count_load_time;
         inkernel_state->channels[2].gate = pit->channels[2].gate;
-        kvm_set_pit(kvm_context, inkernel_state);
+        kvm_set_pit(kvm_state, inkernel_state);
     }
 }
 #else
-static inline void kvm_get_pit_ch2(PITState *pit,
+static inline void kvm_get_pit_ch2(ISADevice *dev,
                                    struct kvm_pit_state *inkernel_state) { }
-static inline void kvm_set_pit_ch2(PITState *pit,
+static inline void kvm_set_pit_ch2(ISADevice *dev,
                                    struct kvm_pit_state *inkernel_state) { }
 #endif
 
@@ -161,7 +164,7 @@ static uint32_t pcspk_ioport_read(void *opaque, uint32_t addr)
     kvm_get_pit_ch2(s->pit, NULL);
 
     s->dummy_refresh_clock ^= (1 << 4);
-    out = pit_get_out(s->pit, 2, qemu_get_clock(vm_clock)) << 5;
+    out = pit_get_out(s->pit, 2, qemu_get_clock_ns(vm_clock)) << 5;
 
     return pit_get_gate(s->pit, 2) | (s->data_on << 1) | s->dummy_refresh_clock | out;
 }
@@ -185,7 +188,7 @@ static void pcspk_ioport_write(void *opaque, uint32_t addr, uint32_t val)
     kvm_set_pit_ch2(s->pit, &inkernel_state);
 }
 
-void pcspk_init(PITState *pit)
+void pcspk_init(ISADevice *pit)
 {
     PCSpkState *s = &pcspk_state;
 
